/****************************************************************************
*  This file is part of the NUTSETUP windows application.
*
*  Copyright (c) 2004 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
*****************************************************************************
*  History:
*
*  11.04.04  mifi   First Version
****************************************************************************/
#define __MAIN_C__
/*==========================================================*/
/*  DEFINE: All Structures and Common Constants             */
/*==========================================================*/
#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <commctrl.h>

#include "resource.h"
#include "version.h"
#include "nutsetupwin.h"

/*==========================================================*/
/*  DEFINE: All Structures and Common Constants             */
/*==========================================================*/
#define APPCLASS              "MF11042004_CLASS"
#define MUTEXNAME             "MF11042004"

#define WIDTH(x)              (x.right - x.left)
#define HEIGHT(x)             (x.bottom - x.top)

#define LIST_VIEW_MAX_HEADER  4

/*==========================================================*/
/*  DEFINE: Prototypes                                      */
/*==========================================================*/

/*==========================================================*/
/*  DEFINE: Definition of all public Data                   */
/*==========================================================*/

/*==========================================================*/
/*  DEFINE: Definition of all local Data                    */
/*==========================================================*/
static HINSTANCE   ghInstApp  = NULL;
static HINSTANCE   ghInstLang = NULL;
static HWND        ghAppWnd   = NULL;

static NM_LISTVIEW gSelectedItem;
static NUTITEM     gNutChangeItem;

static char ListViewHeader[LIST_VIEW_MAX_HEADER][16] = {
  "MAC-Address",
  "IP-Address",
  "Ethernut",
  "Status"
};

/*==========================================================*/
/*  DEFINE: Definition of all local Procedures              */
/*==========================================================*/
/************************************************************/
/*  DoesInstanceExist                                       */
/************************************************************/
static BOOL DoesInstanceExist(int nCmdShow)
{
  HANDLE hMutex;
  HWND hMainWnd;

  //
  //  hPrevInstance-Check Method doesn't work in Win32
  //  reason: hPrevInstance remains NULL anyway
  //  to do:  use Mutex Method instead
  //
  if (hMutex = OpenMutex(MUTEX_ALL_ACCESS, FALSE, MUTEXNAME)) {
    if (hMainWnd = FindWindow(APPCLASS, NULL)) {
      ShowWindow(hMainWnd, nCmdShow);
      BringWindowToTop(hMainWnd);
      return (TRUE);
    }
  } else {
    //
    // If it does not exist, we create the mutex
    //
    hMutex = CreateMutex(NULL, FALSE, MUTEXNAME);
    hMutex = hMutex;
  }

  return (FALSE);
}

/************************************************************/
/*  DrawButtonBitmap                                        */
/************************************************************/
static BOOL DrawButtonBitmap (HINSTANCE hInst, 
                              HWND hDlg, HWND hWnd, HDC hDC, 
                              WORD wBitmap, BOOL fCenter, int ROPCode)
{
  BOOL     fError = FALSE;
  HDC      hMemoryDC;
  HBITMAP  hBitmap;
  BITMAP  bm;
  RECT    rcClient;
  LONG     lHeight, lWidth;

  if (hDC) {
    GetWindowRect(hWnd, (LPRECT)&rcClient);

    lHeight = rcClient.bottom - rcClient.top;
    lWidth  = rcClient.right  - rcClient.left;

    ScreenToClient(hDlg, (POINT FAR*)&rcClient); 
    hMemoryDC = CreateCompatibleDC(hDC);

    if (hMemoryDC) {
      hBitmap   = LoadBitmap(hInst, MAKEINTRESOURCE(wBitmap));
      GetObject(hBitmap,sizeof(BITMAP),(LPSTR)&bm);
      SelectObject(hMemoryDC,hBitmap);

      lHeight = (lHeight - bm.bmHeight) >> 1;
      lWidth  = (lWidth  - bm.bmWidth) >> 1;

      if (fCenter) {
        BitBlt(hDC,
               lWidth, lHeight,
               bm.bmWidth, bm.bmHeight,
               hMemoryDC, 0, 0, ROPCode);
      } else {
        BitBlt(hDC,
               0, 0,
               bm.bmWidth, bm.bmHeight,
               hMemoryDC, 0, 0, ROPCode);
      }

      DeleteObject (hBitmap);
      DeleteDC (hMemoryDC);
    }
  } else {
    fError = TRUE;
  }  

  return (fError);
}

/************************************************************/
/*  AboutProc                                               */
/************************************************************/
static LRESULT CALLBACK AboutProc (HWND hDlg, UINT iMsg,
                                   WPARAM wParam, LPARAM lParam)
{
  switch (iMsg) {

    case WM_INITDIALOG: {
      char szTmp[_MAX_PATH];

      _snprintf(szTmp, sizeof(szTmp), "NUTSetup - V%s", VER_PRODUCTVERSION_STR);
      SetDlgItemText(hDlg, IDC_VERSION, szTmp);

      ShowWindow(hDlg, SW_SHOW);

      SetFocus (hDlg);

      return(TRUE);
      break;
    } /* WM_INITDIALOG */

    case WM_DRAWITEM: {
      LPDRAWITEMSTRUCT pdis = (LPDRAWITEMSTRUCT)lParam;
      HDC  hDC  = pdis->hDC;
      HWND hWnd = pdis->hwndItem;
      
      switch (pdis->CtlID) {
        case IDC_CHIP: {
          if (pdis->itemAction == ODA_DRAWENTIRE) {
            DrawButtonBitmap(ghInstApp, hDlg, hWnd, hDC, IDB_CHIP_SHAPE, FALSE, SRCPAINT);
            DrawButtonBitmap(ghInstApp, hDlg, hWnd, hDC, IDB_CHIP,       FALSE, SRCAND);
          }
          break;
        }
      }
      break;
    } /* WM_DRAWITEM */

    case WM_COMMAND: {
      switch (LOWORD (wParam)) {
        case IDOK: {
          EndDialog(hDlg,0);
          return(FALSE);
          break;                        
        } /* IDOK */
      }
      break;
    } /* WM_COMMAND */
    
    case WM_CLOSE: {
      EndDialog(hDlg,0);
      return(FALSE);
      break;
    } /* WM_CLOSE */
    
  } /* iMsg */

  return (FALSE);
}


/************************************************************/
/*  SetupMainWndControls                                    */
/************************************************************/
static int SetupMainWndControls(HWND hWnd)
{
  int        nError = FALSE;
  char      *pStr;
  LV_COLUMN lvColumn;
  int        nSize, nDelta, nRest;
  int        nIndex;
  HWND       hListView;

  hListView = GetDlgItem(hWnd, IDC_LIST1);

  for(nIndex=0; nIndex<LIST_VIEW_MAX_HEADER; nIndex++) {
    pStr = ListViewHeader[nIndex]; 

    lvColumn.mask       = LVCF_FMT | LVCF_TEXT | LVCF_SUBITEM;
    lvColumn.fmt        = LVCFMT_LEFT;
    lvColumn.pszText    = pStr;
    lvColumn.cchTextMax = sizeof(pStr);
    lvColumn.iSubItem   = nIndex;
    ListView_InsertColumn(hListView, nIndex, &lvColumn);
  }

  for(nIndex=0; nIndex<LIST_VIEW_MAX_HEADER; nIndex++) {
    ListView_SetColumnWidth(hListView, nIndex, LVSCW_AUTOSIZE_USEHEADER); 
  }
  
  nSize  = ListView_GetColumnWidth(hListView, 0);
  nSize += ListView_GetColumnWidth(hListView, 1);
  nSize += ListView_GetColumnWidth(hListView, 2);
  nSize += ListView_GetColumnWidth(hListView, 3);

  nDelta = nSize / 26;  
  ListView_SetColumnWidth(hListView, 0, 5*nDelta); 
  ListView_SetColumnWidth(hListView, 1, 5*nDelta); 
  ListView_SetColumnWidth(hListView, 2, 10*nDelta);
  
  nRest = nSize - (nDelta*20);   
  ListView_SetColumnWidth(hListView, 3, nRest); 

  return (nError);
}

/************************************************************/
/*  DeSelectAll                                             */
/************************************************************/
static void DeSelectAll (HWND hWnd)
{
  int  nIndex;
  int  nMaxItemCount;  
  HWND hListView;
  
  hListView = GetDlgItem(hWnd, IDC_LIST1);

  nMaxItemCount = ListView_GetItemCount(hListView);
  
  for(nIndex=0; nIndex<nMaxItemCount; nIndex++) {
    ListView_SetItemState(hListView, nIndex, 
                          0, 
                          LVIS_CUT | LVIS_DROPHILITED | LVIS_FOCUSED | LVIS_SELECTED);
  }
}

/************************************************************/
/*  DeSelectItem                                            */
/************************************************************/
static void DeSelectItem (HWND hListView)
{
  ListView_SetItemState(hListView, gSelectedItem.iItem, 
                        0, 
                        LVIS_CUT | LVIS_DROPHILITED | LVIS_FOCUSED | LVIS_SELECTED);
}

/************************************************************/
/*                                                          */
/*  ListViewCompareFunc                                     */
/*                                                          */
/*  Sorts the list view control. It is a comparison         */
/*  function. Returns a negative value if the first item    */
/*  should precede the second item, a positive value if the */
/*  first item should follow the second item, and zero if   */
/*  the items are equivalent.                               */
/*                                                          */
/*  lParam1 and lParam2                                     */
/*                                                          */
/*  Item data for the two items (in this case, pointers to  */
/*  application-defined NUTITEM structures)                 */
/*                                                          */
/*  lParamSort                                              */
/*                                                          */
/*  Value specified by the LVM_SORTITEMS message            */
/*  (in this case, the index of the column to sort)         */
/************************************************************/
static int CALLBACK ListViewCompareFunc(LPARAM lParam1, 
                                        LPARAM lParam2, 
                                        LPARAM lParamSort) 
{ 
  NUTITEM *pItem1 = (NUTITEM *) lParam1; 
  NUTITEM *pItem2 = (NUTITEM *) lParam2; 
  int        iCmp;

  // Compare the specified column. 
  iCmp = lstrcmpi(pItem1->aCols[lParamSort], 
                  pItem2->aCols[lParamSort]); 
 
  // Return the result if nonzero, or compare the 
  // first column otherwise. 
  return (iCmp != 0) ? iCmp : lstrcmpi(pItem1->aCols[0], pItem2->aCols[0]); 
} 

/************************************************************/
/*  ChangeProc                                              */
/************************************************************/
static LRESULT CALLBACK ChangeProc (HWND hDlg, UINT iMsg,
                                    WPARAM wParam, LPARAM lParam)
{
  struct in_addr iaEthernut;
  char           *pAddress;  

  switch (iMsg) {

    case WM_INITDIALOG: {
      char szTmp[_MAX_PATH];

      sprintf(szTmp, "Change setting of %02X:%02X:%02X:%02X:%02X:%02X",
              gNutChangeItem.bMACAddress[0], gNutChangeItem.bMACAddress[1],
              gNutChangeItem.bMACAddress[2], gNutChangeItem.bMACAddress[3],
              gNutChangeItem.bMACAddress[4], gNutChangeItem.bMACAddress[5]);     
      SetWindowText(hDlg,szTmp);
      ShowWindow(hDlg, SW_SHOW);

      //
      // Set Name
      //
      SetDlgItemText(hDlg, IDC_NAME, gNutChangeItem.szName);

      //
      // Set IPAddress
      // 
      iaEthernut.s_addr = gNutChangeItem.dAddress;
      pAddress = inet_ntoa(iaEthernut);
      SetDlgItemText(hDlg, IDC_IPADDRESS, pAddress);

      //
      // Set Mask
      // 
      iaEthernut.s_addr = gNutChangeItem.dMask;
      pAddress = inet_ntoa(iaEthernut);
      SetDlgItemText(hDlg, IDC_MASK, pAddress);

      //
      // Set Gateway
      // 
      iaEthernut.s_addr = gNutChangeItem.dGateway;
      pAddress = inet_ntoa(iaEthernut);
      SetDlgItemText(hDlg, IDC_GATEWAY, pAddress);

      SetFocus (hDlg);

      return(TRUE);
      break;
    } /* WM_INITDIALOG */

    case WM_COMMAND: {
      switch (LOWORD (wParam)) {
        case IDOK: {
          char szTmp[_MAX_PATH];

          GetDlgItemText(hDlg, IDC_IPADDRESS, szTmp, _MAX_PATH);
          gNutChangeItem.dAddress = inet_addr(szTmp);

          GetDlgItemText(hDlg, IDC_MASK, szTmp, _MAX_PATH);
          gNutChangeItem.dMask = inet_addr(szTmp);

          GetDlgItemText(hDlg, IDC_GATEWAY, szTmp, _MAX_PATH);
          gNutChangeItem.dGateway = inet_addr(szTmp);

          //
          // Check for correct input
          //
          if( (gNutChangeItem.dAddress != -1) &&
              (gNutChangeItem.dMask    != -1) &&
              (gNutChangeItem.dGateway != -1) ){
            EndDialog(hDlg,1);
          } else {
            EndDialog(hDlg,-1);
          }

          return(FALSE);
          break;                        
        } /* IDOK */
        case IDCANCEL: {
          EndDialog(hDlg,-1);
          return(FALSE);
          break;                        
        } /* IDCANCEL */

      }
      break;
    } /* WM_COMMAND */
    
    case WM_CLOSE: {
      EndDialog(hDlg,-1);
      return(FALSE);
      break;
    } /* WM_CLOSE */
    
  } /* iMsg */

  return (FALSE);
}

/************************************************************/
/*  OnDoubleClick                                           */
/*                                                          */
/*  Processes the NM_DBLCLK notification message.           */
/*                                                          */
/************************************************************/
static void OnDoubleClick (HWND hWnd, HWND hListView, NMITEMACTIVATE *pnmv)
{
  BOOL     fOK;
  int      nResult;
  LV_ITEM lvItem;
  NUTITEM *pItem;

  if (pnmv->iItem != -1) {
    lvItem.mask  = LVIF_PARAM;
    lvItem.iItem = pnmv->iItem;
    fOK = ListView_GetItem(hListView, &lvItem);
    if (fOK) {
      pItem = (NUTITEM *)lvItem.lParam;      

      memcpy(&gNutChangeItem, pItem, sizeof(NUTITEM));

      nResult = DialogBox(ghInstApp, MAKEINTRESOURCE(IDD_CHANGE),
                          hWnd, ChangeProc);
      if (nResult == 1) {
        //
        // Check if we got some new values
        //
        if( (pItem->dAddress != gNutChangeItem.dAddress) ||
            (pItem->dMask    != gNutChangeItem.dMask)    ||
            (pItem->dGateway != gNutChangeItem.dGateway) ){ 
          nut_SetupUpdate(hWnd, lvItem.iItem, pItem, &gNutChangeItem); 
        }
      }
    }    
  }
}

/************************************************************/
/*  OnGetDispInfo                                           */
/*                                                          */
/*  Processes the LVN_GETDISPINFO notification message.     */
/*                                                          */
/************************************************************/
static void OnGetDispInfo(HWND hListView, LV_DISPINFO *pnmv) 
{ 
  NUTITEM *pItem;

  // Provide the item or subitem's text, if requested. 
  if (pnmv->item.mask & LVIF_TEXT) { 
    pItem = (NUTITEM *)(pnmv->item.lParam); 
    lstrcpy(pnmv->item.pszText, 
            pItem->aCols[pnmv->item.iSubItem]); 
  } 
} 

/************************************************************/
/*  OnChanged                                               */
/*                                                          */
/*  Processes the LVN_ITEMCHANGED notification message.     */
/*                                                          */
/************************************************************/
static void OnChanged(HWND hListView, NM_LISTVIEW *pnmv) 
{ 
  if (pnmv->uNewState & LVIS_SELECTED){
    if (gSelectedItem.iItem != -1) {
      //
      // Deselect the old item
      //
      DeSelectItem(hListView);
    } 

    gSelectedItem  = *pnmv;

    //
    // Hilite the new one
    //
    ListView_SetItemState(hListView, gSelectedItem.iItem, 
                          LVIS_DROPHILITED,
                          LVIS_CUT | LVIS_DROPHILITED | LVIS_FOCUSED | LVIS_SELECTED);
  }
} 

/************************************************************/
/*  MainWndProc                                             */
/************************************************************/
static LRESULT CALLBACK MainWndProc(HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
  switch (iMsg) {

    case WM_CREATE:{
      HMENU  hMenu;
      RECT   rDesktop;
      RECT   rApp;
      int    x, y;
      char szTmp[_MAX_PATH];

      //
      // Add new entry to the system menu
      //
      hMenu = GetSystemMenu(hWnd, FALSE);
      AppendMenu(hMenu, MF_SEPARATOR, 0, "");
      AppendMenu(hMenu, MF_STRING, IDC_ABOUT, "About");

      // 
      // Set the window caption
      //
      _snprintf(szTmp, sizeof(szTmp), "Ethernut Configuration  - V%s", VER_PRODUCTVERSION_STR);      
      SetWindowText(hWnd, szTmp);

      //
      // Set the new position in the middle of the desktop
      //
      GetWindowRect(GetDesktopWindow(), &rDesktop);
      GetWindowRect(hWnd, &rApp);
      x = (WIDTH(rDesktop) - WIDTH(rApp)) / 2;
      y = (HEIGHT(rDesktop) - HEIGHT(rApp)) / 2;
      MoveWindow(hWnd, x, y, WIDTH(rApp), HEIGHT(rApp), TRUE);
      break;
    } /* WM_CREATE */


    //
    // 1. Receive WM_CLOSE
    // 
    case WM_CLOSE:{
      //
      // Free all Appliction-Data 
      //
      nut_SetupStop(hWnd);
      break;
    } /* WM_CLOSE */


    //
    // 2. After WM_CLOSE receive WM_DESTROY
    //
    case WM_DESTROY:{
      //
      //  Clear all AppData
      //
      PostQuitMessage(0);
      break;
    } /* WM_DESTROY */


    case WM_DRAWITEM: {
      LPDRAWITEMSTRUCT pdis = (LPDRAWITEMSTRUCT)lParam;
      HDC  hDC  = pdis->hDC;
      HWND hWnd2 = pdis->hwndItem;
      
      switch (pdis->CtlID) {
        case IDC_LOGO: {
          if (pdis->itemAction == ODA_DRAWENTIRE) {
            DrawButtonBitmap(ghInstApp, hWnd, hWnd2, hDC, IDB_LOGO_SHAPE, FALSE, SRCPAINT);
            DrawButtonBitmap(ghInstApp, hWnd, hWnd2, hDC, IDB_LOGO,       FALSE, SRCAND);
          }
          break;
        }
      }
      break;
    } /* WM_DRAWITEM */


    case WM_NOTIFY: { 
      int     idCtrl    = (int)wParam;
      LPNMHDR pnmh      = (LPNMHDR)lParam;
      HWND    hListView = GetDlgItem(hWnd, IDC_LIST1);

      if (idCtrl == IDC_LIST1) {

        switch (pnmh->code) {

          case NM_DBLCLK: {
            OnDoubleClick(hWnd, hListView, (NMITEMACTIVATE *)lParam); 
            break;
          }

          case LVN_GETDISPINFO: {
            OnGetDispInfo(hListView, (LV_DISPINFO *)lParam); 
            break;
          }

          case LVN_ITEMCHANGED: {
            OnChanged(hListView, (NM_LISTVIEW *)lParam);
            break;
          } 

          // Process LVN_COLUMNCLICK to sort items by column. 
          case LVN_COLUMNCLICK: {
            
            //
            // Deselect the old item
            //
            DeSelectAll(hWnd);
    
            #define pnm ((NM_LISTVIEW *)lParam) 
            ListView_SortItems(pnm->hdr.hwndFrom, 
                               ListViewCompareFunc, 
                               (LPARAM)(pnm->iSubItem) ); 
            #undef pnm 
            break; 
          }
        }
      }
      break;
    } /* WM_NOTIFY */


    case WM_PAINT:{
      PAINTSTRUCT ps;
      HDC hDC;

      hDC = BeginPaint(hWnd, &ps);

      EndPaint(hWnd, &ps);
      break;
    } /* WM_PAINT */

    case WM_SYSCOMMAND: {
      switch(wParam) {
        case IDC_ABOUT: {
          SendMessage(hWnd, WM_COMMAND, IDC_LOGO, 0);
          break;
        }
        case SC_SCREENSAVE: {
          return(TRUE); // No ScreenSaver allowed
          break;
        }
      }
      break;
    } /* WM_SYSCOMMAND */

    case WM_COMMAND:{
      switch (LOWORD(wParam)) {

        case IDC_LOGO: {
          DialogBox(ghInstApp, MAKEINTRESOURCE(IDD_ABOUT),
                    hWnd, AboutProc);
          break;
        }
        case IDC_SEARCH: {
          nut_SetupSearch(hWnd);
          break;
        }

        case IDC_QUIT: {
          SendMessage(hWnd, WM_CLOSE, 0, 0); 
          break;
        }

      } /* end switch (LOWORD (wParam)) */
      break;
    } /* WM_COMMAND */


    case SM_EVENT: {      
      if (WSAGETSELECTEVENT(lParam) == FD_READ) {
        nut_SetupRead(hWnd);         
      }
      break;
    } /* SM_EVENT */

  } /* iMsg */

  return (DefWindowProc(hWnd, iMsg, wParam, lParam));
}

/*==========================================================*/
/*  DEFINE: All code exported                               */
/*==========================================================*/
/************************************************************/
/*                       M A I N                            */
/************************************************************/
int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
  int      nError;
  MSG       msg;
  WNDCLASS  wndclass;
  TCHAR   szError[_MAX_PATH];     // Error message string
  WSADATA   WSAData;              // Contains details of the 
                                  // Winsocket implementation
  WORD     wVersion = WINSOCK_VERSION;
  

  if (DoesInstanceExist(nCmdShow)) {
    return (FALSE);
  }

  //
  // Init common control dynamic-link library
  //
  InitCommonControls();

  // Initialize Winsocket 
  if (WSAStartup(wVersion, &WSAData) != 0) {
    wsprintf(szError, TEXT("WSAStartup failed. Error: %d"), WSAGetLastError());
    MessageBox(NULL, szError, TEXT("Error"), MB_OK);
    return (FALSE);
  }

  //
  // Fill in window class structure with 
  // parameters that describe the main window.
  //
  wndclass.style         = CS_HREDRAW | CS_VREDRAW;
  wndclass.lpfnWndProc   = (WNDPROC) MainWndProc;
  wndclass.cbClsExtra    = 0;
  wndclass.cbWndExtra    = DLGWINDOWEXTRA;
  wndclass.hInstance     = hInstance;
  wndclass.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
  wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wndclass.hbrBackground = (HBRUSH) (COLOR_WINDOW);
  wndclass.lpszMenuName  = NULL;
  wndclass.lpszClassName = APPCLASS;

  // Register our new window class
  if (RegisterClass(&wndclass) == 0) {
    return (FALSE);
  }
  
  // Store application instance handle in global variable
  ghInstApp = hInstance;

  // Create a modeless dialog
  ghAppWnd = CreateDialog(ghInstApp, (LPCSTR) IDD_MAINDIALOG, NULL, NULL);
  if (ghAppWnd == NULL) {
    return (FALSE);
  }

  nError = SetupMainWndControls(ghAppWnd);
  if (nError) {
    SendMessage(ghAppWnd, WM_CLOSE, 0, 0);
  }
  
  nError = nut_SetupStart(ghAppWnd);
  if (nError) {
    MessageBox(NULL, "nut_SetupStart", "Error", MB_OK | MB_ICONERROR);
    SendMessage(ghAppWnd, WM_CLOSE, 0, 0);
  } else {
    nut_SetupSearch(ghAppWnd);
  }  

  ShowWindow(ghAppWnd, SW_SHOW);

  // Main message loop
  while (GetMessage(&msg, NULL, 0, 0)) {
    if (!IsDialogMessage(ghAppWnd, &msg)) {
      TranslateMessage(&msg); // Translate virtual key codes 
      DispatchMessage(&msg);  // Dispatches message to window
    }
  }

  WSACleanup();

  return (FALSE);
}
