/****************************************************************************
*  This file is part of the NUTSETUP windows application.
*
*  Copyright (c) 2004 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
*****************************************************************************
*  History:
*
*  11.04.04  mifi   First Version
****************************************************************************/
#define __NUTSETUP_C__
/*==========================================================*/
/*  DEFINE: All Structures and Common Constants             */
/*==========================================================*/
#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <commctrl.h>
#include <process.h>
#include "resource.h"

#include "nutsetupwin.h"
#include "nutsetup.h"

/*==========================================================*/
/*  DEFINE: All Structures and Common Constants             */
/*==========================================================*/

/*==========================================================*/
/*  DEFINE: Prototypes                                      */
/*==========================================================*/

/*==========================================================*/
/*  DEFINE: Definition of all local Data                    */
/*==========================================================*/
static SOCKET   ServerSocket;
static int     nFirstTime = TRUE;

/*==========================================================*/
/*  DEFINE: Definition of all local Procedures              */
/*==========================================================*/

/************************************************************/
/*  SendNutSetup                                            */
/************************************************************/
static void SendNutSetup (NUT_SETUP *pNutSetup)
{
  BOOL         nOptionValue;
  SOCKET        Socket;
  SOCKADDR_IN saDest;

  //
  // Create socket
  //
  Socket = socket(AF_INET, SOCK_DGRAM, 0);
  if (Socket != INVALID_SOCKET) {
    //
    // Change option to BROADCAST
    //
    nOptionValue = TRUE;
    setsockopt(Socket, SOL_SOCKET, SO_BROADCAST, (char *)&nOptionValue, sizeof(BOOL));

    //
    // Set address and port
    //
    saDest.sin_addr.s_addr = inet_addr("255.255.255.255");
    saDest.sin_port        = htons(NUT_SETUP_UDP_PORT);
    saDest.sin_family      = AF_INET;
    
    //
    // Send the packet
    //
    sendto(Socket, (const char *)pNutSetup, sizeof(NUT_SETUP), 0, 
           (const struct sockaddr*)&saDest, sizeof(SOCKADDR_IN));

    //
    // Close the socket
    // 
    closesocket(Socket);
  }
}

/************************************************************/
/*  ClearListView                                           */
/************************************************************/
static void ClearListView (HWND hWnd)
{
  BOOL     fOK;
  int      x;
  int      nIndex;
  int      nMaxItemCount;
  HWND     hListView = GetDlgItem(hWnd, IDC_LIST1);
  LV_ITEM lvItem;
  NUTITEM *pItem;
  
  nMaxItemCount = ListView_GetItemCount(hListView);

  for(nIndex=0; nIndex<nMaxItemCount; nIndex++) {
    lvItem.mask  = LVIF_PARAM;
    lvItem.iItem = nIndex;
    fOK = ListView_GetItem(hListView, &lvItem);
    if (fOK) {
      pItem = (NUTITEM *)lvItem.lParam;
      

      for(x=0; x<4; x++) {
        if (pItem->aCols[x] != NULL) {
          free(pItem->aCols[x]);
          pItem->aCols[x] = NULL;
        } 
      }
      
      free(pItem);
      
      ListView_DeleteItem(hListView, nIndex);  
    }  
  }
}

/************************************************************/
/*  NewNutItem                                              */
/************************************************************/
static NUTITEM* NewNutItem (NUT_SETUP *pNutSetup)
{
  int             nIndex;
  NUTITEM        *pItem;
  char           *pStr; 
  char           *pAddress;
  struct in_addr iaEthernut;   
  char           szTmp[_MAX_PATH];
  DWORD           dVersion; 
  char            bArray[4];
  
  pItem = (NUTITEM *)malloc(sizeof(NUTITEM));
  if (pItem != NULL) {
  
    memset(pItem, 0x00, sizeof(NUTITEM) );
  
    //
    // MAC
    //
    sprintf(szTmp, "%02X:%02X:%02X:%02X:%02X:%02X",
            pNutSetup->bMACAddress[0], pNutSetup->bMACAddress[1], pNutSetup->bMACAddress[2],
            pNutSetup->bMACAddress[3], pNutSetup->bMACAddress[4], pNutSetup->bMACAddress[5]);
    pStr = (LPSTR)malloc(lstrlen(szTmp)+1);
    strcpy(pStr, szTmp);
    pItem->aCols[NUT_ITEM_MAC] = pStr;
    
    //
    // IP-Address
    //
    iaEthernut.s_addr = pNutSetup->dAddress;
    pAddress = inet_ntoa(iaEthernut);
    pStr = (LPSTR)malloc(lstrlen(pAddress)+1);
    strcpy(pStr, pAddress);
    pItem->aCols[NUT_ITEM_IP] = pStr;
    
    //
    // Name
    //
    dVersion  = pNutSetup->dVersion;
    bArray[0] = (BYTE)(dVersion & 0xFF);
    dVersion  = dVersion >> 8; 
    bArray[1] = (BYTE)(dVersion & 0xFF);
    dVersion  = dVersion >> 8; 
    bArray[2] = (BYTE)(dVersion & 0xFF);
    dVersion  = dVersion >> 8; 
    bArray[3] = (BYTE)(dVersion & 0xFF);
    dVersion  = dVersion >> 8; 

    sprintf(szTmp, "%s - V%d.%d.%d.%d", pNutSetup->Name,
            bArray[3], bArray[2], bArray[1], bArray[0]);

    pStr = (LPSTR)malloc(lstrlen(szTmp)+1);
    strcpy(pStr, szTmp);
    pItem->aCols[NUT_ITEM_NAME] = pStr;
    
    //
    // Status
    //
    sprintf(szTmp, "OK");
    pStr = (LPSTR)malloc(lstrlen(szTmp)+1);
    strcpy(pStr, szTmp);
    pItem->aCols[NUT_ITEM_STATUS] = pStr;
    
    for(nIndex=0; nIndex<6; nIndex++) {    
      pItem->bMACAddress[nIndex] = pNutSetup->bMACAddress[nIndex];
    }
    
    pItem->dAddress = pNutSetup->dAddress;  
    pItem->dMask    = pNutSetup->dMask;
    pItem->dGateway = pNutSetup->dGateway;  
    
    memcpy(pItem->szName, pNutSetup->Name, NUT_SETUP_NAME_LEN+1);
  }
  
  return (pItem);
}

/************************************************************/
/*  AddNutItemAgain                                         */
/************************************************************/
static void AddNutItemAgain (HWND hListView, NUTITEM   *pItem,
                             int  nIndex,    NUT_SETUP *pSetup)
{
  char           *pStr;
  char           *pAddress;
  struct in_addr iaEthernut;
  char           szTmp[_MAX_PATH];

  //
  // Update the ListView
  //
  free(pItem->aCols[NUT_ITEM_IP]);
  
  iaEthernut.s_addr = pSetup->dAddress;
  pAddress = inet_ntoa(iaEthernut);
  pStr = (LPSTR)malloc(lstrlen(pAddress)+1);
  strcpy(pStr, pAddress);
  pItem->aCols[NUT_ITEM_IP] = pStr;

  free(pItem->aCols[NUT_ITEM_STATUS]);
  sprintf(szTmp,"OK");
  pStr = (LPSTR)malloc(lstrlen(szTmp)+1);
  strcpy(pStr, szTmp);
  pItem->aCols[NUT_ITEM_STATUS] = pStr;

  ListView_Update (hListView, nIndex);
}

/************************************************************/
/*  AddNutItem                                              */
/************************************************************/
static void AddNutItem (HWND hListView, NUT_SETUP *pNutSetup)
{
  NUTITEM  *pItem;
  LV_ITEM  lvItem;
  
  pItem = NewNutItem(pNutSetup);
  if (pItem != NULL) {
    lvItem.mask       = LVIF_TEXT | LVIF_PARAM | LVIF_STATE;
    lvItem.state      = 0;
    lvItem.stateMask  = 0;
    lvItem.iItem      = 0;
    lvItem.iSubItem   = 0;
    lvItem.pszText    = LPSTR_TEXTCALLBACK;
    lvItem.lParam     = (LPARAM)pItem;
    ListView_InsertItem(hListView, &lvItem);
  }
}

/************************************************************/
/*  HandleResponse                                          */
/************************************************************/
static void HandleResponse (HWND hWnd, NUT_SETUP *pNutSetup)
{
  BOOL     fOK;
  int      nIndex;
  int      nFound;
  int      nMaxItemCount;
  HWND     hListView = GetDlgItem(hWnd, IDC_LIST1);
  LV_ITEM lvItem;
  NUTITEM *pItem;
  
  nMaxItemCount = ListView_GetItemCount(hListView);
  
  //
  // Test if the Setup is always in the list
  //
  nFound = FALSE;
  for(nIndex=0; nIndex<nMaxItemCount; nIndex++) {
    lvItem.mask  = LVIF_PARAM;
    lvItem.iItem = nIndex;
    fOK = ListView_GetItem(hListView, &lvItem);
    if (fOK) {
      pItem = (NUTITEM *)lvItem.lParam;  
      //
      // Check if the MAC address is equal
      //
      if( (pItem->bMACAddress[0] == pNutSetup->bMACAddress[0]) &&
          (pItem->bMACAddress[1] == pNutSetup->bMACAddress[1]) &&
          (pItem->bMACAddress[2] == pNutSetup->bMACAddress[2]) &&
          (pItem->bMACAddress[3] == pNutSetup->bMACAddress[3]) &&
          (pItem->bMACAddress[4] == pNutSetup->bMACAddress[4]) &&
          (pItem->bMACAddress[5] == pNutSetup->bMACAddress[5]) ){
        nFound = TRUE;
        break;    
      }    
    }    
  }

  if (nFound == FALSE) {
    //
    // We can put the new Setup in the list
    //
    AddNutItem(hListView, pNutSetup);
  } else {
    //
    // Add the item again
    //
    AddNutItemAgain(hListView, pItem, lvItem.iItem, pNutSetup); 
  }  
}

/*==========================================================*/
/*  DEFINE: All code exported                               */
/*==========================================================*/
/************************************************************/
/*  nut_SetupStart                                          */
/************************************************************/
int nut_SetupStart (HWND hWnd)
{
  int           nError = TRUE;
  int           nRet;
  SOCKADDR_IN  saSource;
  
  if (nFirstTime == TRUE) {
    nFirstTime = FALSE;

    ServerSocket = socket(AF_INET, SOCK_DGRAM, 0);

    //
    // Set address and port
    //
    saSource.sin_addr.s_addr = INADDR_ANY;
    saSource.sin_port        = htons(NUT_SETUP_UDP_PORT);
    saSource.sin_family      = AF_INET;

    nRet = bind(ServerSocket, (const struct sockaddr*)&saSource, sizeof(SOCKADDR_IN));
    
    nRet = WSAAsyncSelect(ServerSocket, hWnd, SM_EVENT, FD_READ);
    if (nRet != SOCKET_ERROR) {
      nError = FALSE;
    }                          
  }

  return (nError);
}

/************************************************************/
/*  nut_SetupStop                                           */
/************************************************************/
void nut_SetupStop (HWND hWnd)
{
  if (nFirstTime == FALSE) {
    closesocket(ServerSocket);
  }
}

/************************************************************/
/*  nut_SetupRead                                           */
/*                                                          */
/*  We have receive some data, read it out.                 */
/*                                                          */
/************************************************************/
void nut_SetupRead (HWND hWnd)
{
  int          nRet;
  int          nAddressLen;
  SOCKADDR_IN saSource;
  NUT_SETUP     NutSetup;
  
  nAddressLen = sizeof(SOCKADDR_IN);
  nRet = recvfrom(ServerSocket, 
                  (char *)&NutSetup, 
                  sizeof(NUT_SETUP), 
                  0,
                  (struct sockaddr*)&saSource,
                  &nAddressLen);

  if( (nRet            != SOCKET_ERROR)        &&
      (NutSetup.wMagic == NUT_SETUP_MAGIC)     &&
      (NutSetup.wSize  == sizeof(NUT_SETUP))   &&
      (NutSetup.bMode  == NUT_SETUP_RESPONSE) ){    
    HandleResponse(hWnd, &NutSetup);      
  }  
}

/************************************************************/
/*  nut_SetupSearch                                         */
/*                                                          */
/*  Send a SEARCH_REQUEST to the network                    */
/*                                                          */
/************************************************************/
void nut_SetupSearch (HWND hWnd)
{
  BOOL         nOptionValue;
  SOCKET        Socket;
  SOCKADDR_IN saDest;
  NUT_SETUP     NutSetup;  

  //
  // Create socket
  //
  Socket = socket(AF_INET, SOCK_DGRAM, 0);
  if (Socket != INVALID_SOCKET) {
  
    //
    // First we must delete all the LitViewItems,
    // we must do it twice, to delete the last one too.
    //
    ClearListView(hWnd);
    ClearListView(hWnd);
  
    //
    // Change option to BROADCAST
    //
    nOptionValue = TRUE;
    setsockopt(Socket, SOL_SOCKET, SO_BROADCAST, (char *)&nOptionValue, sizeof(BOOL));

    //
    // Set address and port
    //
    saDest.sin_addr.s_addr = inet_addr("255.255.255.255");
    saDest.sin_port        = htons(NUT_SETUP_UDP_PORT);
    saDest.sin_family      = AF_INET;

    //
    // Fill the packet
    // 
    memset(&NutSetup, 0x00, sizeof(NUT_SETUP));
    NutSetup.wMagic = NUT_SETUP_MAGIC;
    NutSetup.wSize  = sizeof(NUT_SETUP);
    NutSetup.bMode  = NUT_SETUP_REQUEST;
    
    //
    // Send the packet
    //
    sendto(Socket, (const char *)&NutSetup, sizeof(NUT_SETUP), 0, 
           (const struct sockaddr*)&saDest, sizeof(SOCKADDR_IN));

    //
    // Close the socket
    // 
    closesocket(Socket);
  }  
}

/************************************************************/
/*  nut_SetupUpdate                                         */
/*                                                          */
/*  Update a ListViewItem, set the new state to Reboot.     */
/*  After this we must update the Ethernut.                 */
/*  Ethernut will make a reboot after the update.           */
/*                                                          */
/************************************************************/
void nut_SetupUpdate (HWND hWnd, int nItem, NUTITEM *pOld, NUTITEM *pNew)
{
  HWND            hListView = GetDlgItem(hWnd, IDC_LIST1);
  char           *pStr;
  char           *pAddress;
  struct in_addr iaEthernut;
  char           szTmp[_MAX_PATH];
  
  int             nIndex;
  BOOL            nOptionValue;
  SOCKET           Socket;
  SOCKADDR_IN    saDest;
  NUT_SETUP        NutSetup;  
  
  //
  // Update the ListView
  //
  free(pOld->aCols[NUT_ITEM_IP]);
  
  iaEthernut.s_addr = pNew->dAddress;
  pAddress = inet_ntoa(iaEthernut);
  pStr = (LPSTR)malloc(lstrlen(pAddress)+1);
  strcpy(pStr, pAddress);
  pOld->aCols[NUT_ITEM_IP] = pStr;

  pOld->dAddress = pNew->dAddress;
  pOld->dMask    = pNew->dMask;
  pOld->dGateway = pNew->dGateway;

  pOld->nStatus  = pNew->nStatus;

  free(pOld->aCols[NUT_ITEM_STATUS]);
  sprintf(szTmp,"Reboot...");
  pStr = (LPSTR)malloc(lstrlen(szTmp)+1);
  strcpy(pStr, szTmp);
  pOld->aCols[NUT_ITEM_STATUS] = pStr;

  ListView_Update (hListView, nItem);
  
  //
  // Send the Update to the Ethernut
  //
  
  // Create socket
  Socket = socket(AF_INET, SOCK_DGRAM, 0);
  if (Socket != INVALID_SOCKET) {
    
    //
    // Change option to BROADCAST
    //
    nOptionValue = TRUE;
    setsockopt(Socket, SOL_SOCKET, SO_BROADCAST, (char *)&nOptionValue, sizeof(BOOL));

    //
    // Set address and port
    //
    saDest.sin_addr.s_addr = inet_addr("255.255.255.255");
    saDest.sin_port        = htons(NUT_SETUP_UDP_PORT);
    saDest.sin_family      = AF_INET;

    //
    // Fill the packet
    // 
    memset(&NutSetup, 0x00, sizeof(NUT_SETUP));
    NutSetup.wMagic = NUT_SETUP_MAGIC;
    NutSetup.wSize  = sizeof(NUT_SETUP);
    NutSetup.bMode  = NUT_SETUP_SET;
    
    for(nIndex=0; nIndex<6; nIndex++) {
      NutSetup.bMACAddress[nIndex] = pNew->bMACAddress[nIndex];
    }
    
    NutSetup.dAddress = pNew->dAddress;
    NutSetup.dMask    = pNew->dMask;
    NutSetup.dGateway = pNew->dGateway;      
    
    //
    // Send the packet
    //
    sendto(Socket, (const char *)&NutSetup, sizeof(NUT_SETUP), 0, 
           (const struct sockaddr*)&saDest, sizeof(SOCKADDR_IN));

    //
    // Close the socket
    // 
    closesocket(Socket);
  }  
}
